/**
 * @file 	DiscreteKernel.java
 * 
 * @date 	Jan 16, 2005
 * @author 	Alexandre Bouchard
 */

package ca.mcgill.lmp.util.kernels;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import ca.mcgill.lmp.MeasurableFunction;
import ca.mcgill.lmp.MeasurableSet;
import ca.mcgill.lmp.Measure;
import ca.mcgill.lmp.State;
import ca.mcgill.lmp.TransitionKernel;
import ca.mcgill.lmp.util.DiscreteSet;

/**
 *
 */
public class DiscreteKernel implements TransitionKernel
{
    private Map probabilities = new HashMap();
    private MeasurableSet X;
    private Set states = new HashSet();
    
    public DiscreteKernel()
    {     
    }

    /**
     * @return Returns the x.
     */
    public MeasurableSet getX()
    {
        return X;
    }
    
    /**
     * @param x The x to set.
     */
    public void setX(MeasurableSet x)
    {
        X = x;
    }
    
    /**
     * @param s
     * @return
     */
    public Measure getTransitionMeasure(State s)
    {
        return new ApproximationMeasure(s);
    }
    
    /**
     * @param source
     * @param destination
     * @param probability
     */
    public void setProbability(State source, State destination, double probability)
    {
        states.add(source);
        states.add(destination);
        List key = getKey(source, destination);
        probabilities.put(key, new Double(probability));
    }
    
    /**
     * @param source
     * @param destination
     * @return
     */
    private List getKey(State source, State destination)
    {
        List key = new ArrayList();
        key.add(source);
        key.add(destination);
        return key;
    }
    
    /**
     * @param source
     * @param destination
     * @return
     */
    private double Pr(State source, State destination)
    {
        List key = getKey(source, destination);
        if (probabilities.containsKey(key))
        {
            return ((Double) probabilities.get(key)).doubleValue();
        }
        return 0.0;        
    }
    
    /**
     * @param source
     * @param destination
     * @return
     */
    private double Pr(State source, MeasurableSet destination)
    {
        Iterator i = states.iterator();
        double partialSum = 0;
        while (i.hasNext())
        {
            State currentState = (State) i.next();
            if (destination.contains(currentState))
            {
                partialSum = partialSum + Pr(source, currentState);
            }
        }
        return partialSum;        
    }
    
    public String toString()
    {
        StringBuffer buffer = new StringBuffer();
        Iterator i = probabilities.keySet().iterator();
        while (i.hasNext())
        {
            List currentKey = (List) i.next();
            buffer.append(currentKey.get(0).toString() + " -> " +
                    currentKey.get(1).toString());
            double probability = ((Double) probabilities.get(currentKey)).doubleValue();
            buffer.append("  (" + probability + ")\n");
        }
        return buffer.toString();
    }

    /**
     * @param B
     * @return
     */
    public MeasurableFunction getTransitionFunction(MeasurableSet B)
    {
        return new ApproximationTransitionFunction(B);
    }

    class ApproximationMeasure implements Measure
    {
        private State source;
        
        public ApproximationMeasure(State s)
        {
            source = s;
        }

        /**
         * @param B
         * @return
         */
        public double measure(MeasurableSet destination)
        {
            return Pr(source, destination);
        }
        
        public double measure(State destination)
        {
            return Pr(source, destination);
        }

        /**
         * @param B
         * @return
         */
        public State sampleIn(MeasurableSet B)
        {
            throw new RuntimeException("ApproximationMeasure.sampleIn: not implemented.");
        }

        /**
         * @return
         */
        public MeasurableSet getX()
        {
            return X;
        } 
        
        /**
         * @return
         */
        public State sample()
        {
            throw new RuntimeException("ApproximationMeasure.sampleIn: not implemented.");
        }           
    }
    
    class ApproximationTransitionFunction implements MeasurableFunction
    {
        /**
         * Comment for <code>destination</code>
         */
        private MeasurableSet destination;
        
        /**
         * @param B
         */
        public ApproximationTransitionFunction(MeasurableSet B)
        {
            destination = B;
        }
        
        /**
         * @param state
         */
        public ApproximationTransitionFunction(State state)
        {
            DiscreteSet destination = new DiscreteSet();
            destination.add(state);
            this.destination = destination;
        }

        /**
         * @param s
         * @return
         */
        public double evaluateAt(State source)
        {
            return Pr(source, destination);
        }
        
    }
}
